@@ -1049,12 +1049,10 @@ KdGetOptions (InputOption **options, char *string)
     if (strchr(string, '='))
     {
         tam_key = (strchr(string, '=') - string);
-        key = malloc(tam_key + 1);
+        key = strndup(string, tam_key);
         if (!key)
             goto out;
 
-        strncpy(key, string, tam_key);
-        key[tam_key] = '\0';
         value = strdup(strchr(string, '=') + 1);
         if (!value)
             goto out;
@@ -2148,12 +2146,6 @@ KdCursorOffScreen(ScreenPtr *ppScreen, int *x, int *y)
 static void
 KdCrossScreen(ScreenPtr pScreen, Bool entering)
 {
-#ifndef XIPAQ
-    if (entering)
-	KdEnableScreen (pScreen);
-    else
-	KdDisableScreen (pScreen);
-#endif
 }
 
 int KdCurScreen;	/* current event screen */
