@@ -120,7 +120,7 @@ FreeList(const char ***list, int *lines)
     int i;
 
     for (i = 0; i < *lines; i++) {
-	free((*list)[i]);
+	free((char *)((*list)[i]));
     }
     free(*list);
     *list = NULL;
@@ -255,9 +255,9 @@ listPossibleVideoDrivers(char *matches[], int nmatches)
 	    matches[i++] = xnfstrdup(sbusDriver);
     }
 #endif
-
+#ifdef XSERVER_LIBPCIACCESS
     i = xf86PciMatchDriver(matches, nmatches);
-
+#endif
     /* Fallback to platform default hardware */
     if (i < (nmatches - 1)) {
 #if defined(__i386__) || defined(__amd64__) || defined(__hurd__)
@@ -267,6 +267,7 @@ listPossibleVideoDrivers(char *matches[], int nmatches)
 #endif
     }
 
+#if !defined(sun)
     /* Fallback to platform default frame buffer driver */
     if (i < (nmatches - 1)) {
 #if !defined(__linux__) && defined(__sparc__)
@@ -275,6 +276,7 @@ listPossibleVideoDrivers(char *matches[], int nmatches)
 	matches[i++] = xnfstrdup("fbdev");
 #endif
     }
+#endif /* !sun */
 }
 
 /* copy a screen section and enter the desired driver
@@ -282,21 +284,31 @@ listPossibleVideoDrivers(char *matches[], int nmatches)
 static Bool
 copyScreen(confScreenPtr oscreen, GDevPtr odev, int i, char *driver)
 {
+    confScreenPtr nscreen;
     GDevPtr cptr = NULL;
 
-    xf86ConfigLayout.screens[i].screen = xnfcalloc(1, sizeof(confScreenRec));
-    if(!xf86ConfigLayout.screens[i].screen)
+    nscreen = malloc(sizeof(confScreenRec));
+    if (!nscreen)
         return FALSE;
-    memcpy(xf86ConfigLayout.screens[i].screen, oscreen, sizeof(confScreenRec));
+    memcpy(nscreen, oscreen, sizeof(confScreenRec));
 
-    cptr = calloc(1, sizeof(GDevRec));
-    if (!cptr)
+    cptr = malloc(sizeof(GDevRec));
+    if (!cptr) {
+        free(nscreen);
         return FALSE;
+    }
     memcpy(cptr, odev, sizeof(GDevRec));
 
-    cptr->identifier = Xprintf("Autoconfigured Video Device %s", driver);
+    if (asprintf(&cptr->identifier, "Autoconfigured Video Device %s", driver)
+        == -1) {
+        free(cptr);
+        free(nscreen);
+        return FALSE;
+    }
     cptr->driver = driver;
 
+    xf86ConfigLayout.screens[i].screen = nscreen;
+
     /* now associate the new driver entry with the new screen entry */
     xf86ConfigLayout.screens[i].screen->device = cptr;
     cptr->myScreenSection = xf86ConfigLayout.screens[i].screen;
