@@ -43,6 +43,7 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "inputstr.h"
 #include "opaque.h"
 #include "property.h"
+#include "scrnintstr.h"
 #define	XKBSRV_NEED_FILE_FUNCS
 #include <xkbsrv.h>
 #include "xkbgeom.h"
@@ -84,11 +85,11 @@ typedef struct	_SrvXkmInfo {
 /***====================================================================***/
 
 #ifndef XKB_DFLT_RULES_PROP
-#define	XKB_DFLT_RULES_PROP	True
+#define	XKB_DFLT_RULES_PROP	TRUE
 #endif
 
-char	*		XkbBaseDirectory=	XKB_BASE_DIRECTORY;
-char	*		XkbBinDirectory=	XKB_BIN_DIRECTORY;
+const char *		XkbBaseDirectory=	XKB_BASE_DIRECTORY;
+const char *		XkbBinDirectory=	XKB_BIN_DIRECTORY;
 static int	 	XkbWantAccessX=		0;	
 
 static char *		XkbRulesDflt=		NULL;
@@ -116,22 +117,11 @@ static Bool		XkbWantRulesProp=	XKB_DFLT_RULES_PROP;
 void
 XkbGetRulesDflts(XkbRMLVOSet *rmlvo)
 {
-    if (XkbRulesDflt)   rmlvo->rules = XkbRulesDflt;
-    else                rmlvo->rules = XKB_DFLT_RULES;
-    if (XkbModelDflt)	rmlvo->model= XkbModelDflt;
-    else		rmlvo->model= XKB_DFLT_MODEL;
-    if (XkbLayoutDflt)	rmlvo->layout= XkbLayoutDflt;
-    else		rmlvo->layout= XKB_DFLT_LAYOUT;
-    if (XkbVariantDflt)	rmlvo->variant= XkbVariantDflt;
-    else		rmlvo->variant= XKB_DFLT_VARIANT;
-    if (XkbOptionsDflt)	rmlvo->options= XkbOptionsDflt;
-    else		rmlvo->options= XKB_DFLT_OPTIONS;
-
-    rmlvo->rules = strdup(rmlvo->rules);
-    rmlvo->model = strdup(rmlvo->model);
-    rmlvo->layout = strdup(rmlvo->layout);
-    rmlvo->variant = strdup(rmlvo->variant);
-    rmlvo->options = strdup(rmlvo->options);
+    rmlvo->rules  = strdup(XkbRulesDflt   ? XkbRulesDflt   : XKB_DFLT_RULES);
+    rmlvo->model  = strdup(XkbModelDflt   ? XkbModelDflt   : XKB_DFLT_MODEL);
+    rmlvo->layout = strdup(XkbLayoutDflt  ? XkbLayoutDflt  : XKB_DFLT_LAYOUT);
+    rmlvo->variant= strdup(XkbVariantDflt ? XkbVariantDflt : XKB_DFLT_VARIANT);
+    rmlvo->options= strdup(XkbOptionsDflt ? XkbOptionsDflt : XKB_DFLT_OPTIONS);
 }
 
 void
@@ -140,14 +130,14 @@ XkbFreeRMLVOSet(XkbRMLVOSet *rmlvo, Bool freeRMLVO)
     if (!rmlvo)
         return;
 
-    xfree(rmlvo->rules);
-    xfree(rmlvo->model);
-    xfree(rmlvo->layout);
-    xfree(rmlvo->variant);
-    xfree(rmlvo->options);
+    free(rmlvo->rules);
+    free(rmlvo->model);
+    free(rmlvo->layout);
+    free(rmlvo->variant);
+    free(rmlvo->options);
 
     if (freeRMLVO)
-        xfree(rmlvo);
+        free(rmlvo);
     else
         memset(rmlvo, 0, sizeof(XkbRMLVOSet));
 }
@@ -165,20 +155,20 @@ char *			pval;
     len+= (XkbVariantUsed?strlen(XkbVariantUsed):0);
     len+= (XkbOptionsUsed?strlen(XkbOptionsUsed):0);
     if (len<1)
-	return True;
+	return TRUE;
 
     len+= 5; /* trailing NULs */
 
     name= MakeAtom(_XKB_RF_NAMES_PROP_ATOM,strlen(_XKB_RF_NAMES_PROP_ATOM),1);
     if (name==None) {
 	ErrorF("[xkb] Atom error: %s not created\n",_XKB_RF_NAMES_PROP_ATOM);
-	return True;
+	return TRUE;
     }
-    pval= (char*) xalloc(len);
+    pval= (char*) malloc(len);
     if (!pval) {
 	ErrorF("[xkb] Allocation error: %s proprerty not created\n",
 						_XKB_RF_NAMES_PROP_ATOM);
-	return True;
+	return TRUE;
     }
     out= 0;
     if (XkbRulesUsed) {
@@ -210,30 +200,25 @@ char *			pval;
 	ErrorF("[xkb] Internal Error! bad size (%d!=%d) for _XKB_RULES_NAMES\n",
 								out,len);
     }
-    dixChangeWindowProperty(serverClient, WindowTable[0], name, XA_STRING, 8,
-			    PropModeReplace, len, pval, True);
-    xfree(pval);
-    return True;
+    dixChangeWindowProperty(serverClient, screenInfo.screens[0]->root, name, XA_STRING, 8,
+			    PropModeReplace, len, pval, TRUE);
+    free(pval);
+    return TRUE;
 }
 
 static void
 XkbSetRulesUsed(XkbRMLVOSet *rmlvo)
 {
-    if (XkbRulesUsed)
-        xfree(XkbRulesUsed);
-    XkbRulesUsed= (rmlvo->rules?_XkbDupString(rmlvo->rules):NULL);
-    if (XkbModelUsed)
-	xfree(XkbModelUsed);
-    XkbModelUsed= (rmlvo->model?_XkbDupString(rmlvo->model):NULL);
-    if (XkbLayoutUsed)
-	xfree(XkbLayoutUsed);
-    XkbLayoutUsed= (rmlvo->layout?_XkbDupString(rmlvo->layout):NULL);
-    if (XkbVariantUsed)
-	xfree(XkbVariantUsed);
-    XkbVariantUsed= (rmlvo->variant?_XkbDupString(rmlvo->variant):NULL);
-    if (XkbOptionsUsed)
-	xfree(XkbOptionsUsed);
-    XkbOptionsUsed= (rmlvo->options?_XkbDupString(rmlvo->options):NULL);
+    free(XkbRulesUsed);
+    XkbRulesUsed= (rmlvo->rules?Xstrdup(rmlvo->rules):NULL);
+    free(XkbModelUsed);
+    XkbModelUsed= (rmlvo->model?Xstrdup(rmlvo->model):NULL);
+    free(XkbLayoutUsed);
+    XkbLayoutUsed= (rmlvo->layout?Xstrdup(rmlvo->layout):NULL);
+    free(XkbVariantUsed);
+    XkbVariantUsed= (rmlvo->variant?Xstrdup(rmlvo->variant):NULL);
+    free(XkbOptionsUsed);
+    XkbOptionsUsed= (rmlvo->options?Xstrdup(rmlvo->options):NULL);
     if (XkbWantRulesProp)
 	QueueWorkProc(XkbWriteRulesProp,NULL,NULL);
     return;
@@ -243,29 +228,24 @@ void
 XkbSetRulesDflts(XkbRMLVOSet *rmlvo)
 {
     if (rmlvo->rules) {
-        if (XkbRulesDflt)
-	    xfree(XkbRulesDflt);
-        XkbRulesDflt= _XkbDupString(rmlvo->rules);
+        free(XkbRulesDflt);
+        XkbRulesDflt= Xstrdup(rmlvo->rules);
     }
     if (rmlvo->model) {
-	if (XkbModelDflt)
-	    xfree(XkbModelDflt);
-	XkbModelDflt= _XkbDupString(rmlvo->model);
+	free(XkbModelDflt);
+	XkbModelDflt= Xstrdup(rmlvo->model);
     }
     if (rmlvo->layout) {
-	if (XkbLayoutDflt)
-	    xfree(XkbLayoutDflt);
-	XkbLayoutDflt= _XkbDupString(rmlvo->layout);
+	free(XkbLayoutDflt);
+	XkbLayoutDflt= Xstrdup(rmlvo->layout);
     }
     if (rmlvo->variant) {
-	if (XkbVariantDflt)
-	    xfree(XkbVariantDflt);
-	XkbVariantDflt= _XkbDupString(rmlvo->variant);
+	free(XkbVariantDflt);
+	XkbVariantDflt= Xstrdup(rmlvo->variant);
     }
     if (rmlvo->options) {
-	if (XkbOptionsDflt)
-	    xfree(XkbOptionsDflt);
-	XkbOptionsDflt= _XkbDupString(rmlvo->options);
+	free(XkbOptionsDflt);
+	XkbOptionsDflt= Xstrdup(rmlvo->options);
     }
     return;
 }
@@ -273,18 +253,18 @@ XkbSetRulesDflts(XkbRMLVOSet *rmlvo)
 void
 XkbDeleteRulesDflts(void)
 {
-    xfree(XkbRulesDflt);
+    free(XkbRulesDflt);
     XkbRulesDflt = NULL;
-    xfree(XkbModelDflt);
+    free(XkbModelDflt);
     XkbModelDflt = NULL;
-    xfree(XkbLayoutDflt);
+    free(XkbLayoutDflt);
     XkbLayoutDflt = NULL;
-    xfree(XkbVariantDflt);
+    free(XkbVariantDflt);
     XkbVariantDflt = NULL;
-    xfree(XkbOptionsDflt);
+    free(XkbOptionsDflt);
     XkbOptionsDflt = NULL;
 
-    XkbFreeKeyboard(xkb_cached_map, XkbAllComponentsMask, True);
+    XkbFreeKeyboard(xkb_cached_map, XkbAllComponentsMask, TRUE);
     xkb_cached_map = NULL;
 }
 
@@ -312,17 +292,17 @@ static Bool
 XkbInitKeyTypes(XkbDescPtr xkb)
 {
     if (xkb->defined & XkmTypesMask)
-        return True;
+        return TRUE;
 
     initTypeNames(NULL);
     if (XkbAllocClientMap(xkb,XkbKeyTypesMask,num_dflt_types)!=Success)
-	return False;
+	return FALSE;
     if (XkbCopyKeyTypes(dflt_types,xkb->map->types,num_dflt_types)!=
     								 Success) {
-	return False;
+	return FALSE;
     }
     xkb->map->size_types= xkb->map->num_types= num_dflt_types;
-    return True;
+    return TRUE;
 }
 
 static void
@@ -341,7 +321,7 @@ register int 	i;
 XkbCompatMapPtr	compat;
 
     if (xkb->defined & XkmCompatMapMask)
-        return True;
+        return TRUE;
 
     if (XkbAllocCompatMap(xkb,XkbAllCompatMask,num_dfltSI)!=Success)
 	return BadAlloc;
@@ -515,20 +495,20 @@ InitKeyboardDeviceStruct(DeviceIntPtr dev, XkbRMLVOSet *rmlvo,
     memset(&changes, 0, sizeof(changes));
     XkbSetCauseUnknown(&cause);
 
-    dev->key = xcalloc(1, sizeof(*dev->key));
+    dev->key = calloc(1, sizeof(*dev->key));
     if (!dev->key) {
         ErrorF("XKB: Failed to allocate key class\n");
-        return False;
+        return FALSE;
     }
     dev->key->sourceid = dev->id;
 
-    dev->kbdfeed = xcalloc(1, sizeof(*dev->kbdfeed));
+    dev->kbdfeed = calloc(1, sizeof(*dev->kbdfeed));
     if (!dev->kbdfeed) {
         ErrorF("XKB: Failed to allocate key feedback class\n");
         goto unwind_key;
     }
 
-    xkbi = xcalloc(1, sizeof(*xkbi));
+    xkbi = calloc(1, sizeof(*xkbi));
     if (!xkbi) {
         ErrorF("XKB: Failed to allocate XKB info\n");
         goto unwind_kbdfeed;
@@ -536,7 +516,7 @@ InitKeyboardDeviceStruct(DeviceIntPtr dev, XkbRMLVOSet *rmlvo,
     dev->key->xkbInfo = xkbi;
 
     if (xkb_cached_map && !XkbCompareUsedRMLVO(rmlvo)) {
-        XkbFreeKeyboard(xkb_cached_map, XkbAllComponentsMask, True);
+        XkbFreeKeyboard(xkb_cached_map, XkbAllComponentsMask, TRUE);
         xkb_cached_map = NULL;
     }
 
@@ -620,13 +600,13 @@ InitKeyboardDeviceStruct(DeviceIntPtr dev, XkbRMLVOSet *rmlvo,
 unwind_desc:
     XkbFreeKeyboard(xkb, 0, TRUE);
 unwind_info:
-    xfree(xkbi);
+    free(xkbi);
     dev->key->xkbInfo = NULL;
 unwind_kbdfeed:
-    xfree(dev->kbdfeed);
+    free(dev->kbdfeed);
     dev->kbdfeed = NULL;
 unwind_key:
-    xfree(dev->key);
+    free(dev->key);
     dev->key = NULL;
     return FALSE;
 }
@@ -644,10 +624,8 @@ unwind_key:
 void
 XkbFreeInfo(XkbSrvInfoPtr xkbi)
 {
-    if (xkbi->radioGroups) {
-	xfree(xkbi->radioGroups);
-	xkbi->radioGroups= NULL;
-    }
+    free(xkbi->radioGroups);
+    xkbi->radioGroups = NULL;
     if (xkbi->mouseKeyTimer) {
 	TimerFree(xkbi->mouseKeyTimer);
 	xkbi->mouseKeyTimer= NULL;
@@ -674,10 +652,10 @@ XkbFreeInfo(XkbSrvInfoPtr xkbi)
 	xkbi->beepTimer= NULL;
     }
     if (xkbi->desc) {
-	XkbFreeKeyboard(xkbi->desc,XkbAllComponentsMask,True);
+	XkbFreeKeyboard(xkbi->desc,XkbAllComponentsMask,TRUE);
 	xkbi->desc= NULL;
     }
-    xfree(xkbi);
+    free(xkbi);
     return;
 }
 
